;; Eric Schulte
;; 2010-03-23
;;
;; WYSWYG, html mail composition using org-mode
;;
;; For mail composed using the orgstruct-mode minor mode, this
;; provides a function for converting all or part of your mail buffer
;; to embedded html as exported by org-mode.  Call `org-mml-htmlize'
;; in a message buffer to convert either the active region or the
;; entire buffer to html.
;;

(defun org-mml-htmlize (arg)
  (interactive "P") ;; later just do <pre> wrap if prefix arg
  (let* ((region-p (org-region-active-p))
         (html-start (or (and region-p (region-beginning))
                         (save-excursion
                           (goto-char (point-min))
                           (search-forward mail-header-separator)
                           (point))))
         (html-end (or (and region-p (region-end))
                       ;; TODO: should catch signature...
                       (point-max)))
         (body (buffer-substring html-start html-end))
         (tmp-file (make-temp-name (expand-file-name "mail" "/tmp/")))
         ;; because we probably don't want to skip part of our mail
         (org-export-skip-text-before-1st-heading nil)
         ;; makes the replies with ">"s look nicer
         (org-export-preserve-breaks t)
         (html (save-excursion
                 (with-temp-buffer
                   (insert body)
                   (write-file tmp-file)
                   ;; convert to html -- mimicing org-run-like-in-org-mode
                   (eval (list 'let org-local-vars
                               (list 'org-export-as-html nil nil nil ''string t)))))))
    (delete-region html-start html-end)
    (save-excursion
      (goto-char html-start)
      (insert
       (format
        "\n<#multipart type=alternative>\n<#part type=text/html>%s<#/multipart>\n"
        html)))))
